#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include "nrutil.h"

/*RUN SPECIFICATIONS*/
#define	DpDIM  		70	/*number of sections used*/
#define	DpMIN      	0.020 /*lower bound diameter in um, >=.1*/
#define DpMAX		3 /*upper bound diameter in um */
#define	dt		10.000e0  /*time step in seconds*/
#define dtOUT		20.0 /*interval (sec) for saving distrib data to file*/
#define rateOUT		10.0 /*factor (of dtOUT) for saving rate data*/
#define	endt		40000.0
#define ScMAX		0.05/*0.05~1%max,0.01~.5%max, 0.1~2.5%*/
#define profFLAG	-1 /*+1=ascending profile data; -1=descending profile*/

/*INPUT/OUTPUT FILE SPECIFIERS*/
#define INsuffix	".inp.MH233"
#define OUTsuffix	".out.MH233" /*suffix for output files*/
#define	maxLINE		200
#define maxWORD		10
#define keyTYPE		"ParticleType" /*keyword for particle pop'n */
#define	METfile		"metfile" /* file name of met data*/
#define	MASSfile	"massfile" /* file name of Mp data*/
#define SALTfile	"saltfile.dat" /* file name of sea spray Np distrib. */
#define INITfile	"initfile"
#define CvOUT		"Cvapor" /*prefix for gas concentration output */
#define NpOUT		"Npart"  /*prefix for number concentration output */
#define MpOUT		"Mpart"  /*prefix for mass concentration output */
#define DpOUT		"Dpart"  /*prefix for ambient diameters */
#define MetOUT		"Meteo"  /*prefix for meteorological data output */
#define mzsumtest	"mzsumtest"
#define ratetest	"ratetest"

/*TYPE AND SPECIES IDENTIFIERS*/
#define SpDIM 		12	/*total number of vapor/mass species included*/
#define  h2oSP		0
#define  dmsSP		1
#define  so2SP		2
#define  h2so4SP	3
#define  blkCSP		8
#define	 naclSP		6
#define  orgCSP		7
#define  nh42so4SP	4
#define  nh4hso4SP	5
#define  nh4no3SP       9
#define  nano3SP        10
#define  hno3SP         11
#define  airSP		-10
#define TpDIM 		2	/*total number of particle types included*/
#define  sulfateTP	2
#define  saltTP		0
#define  plumeTP	1
#define  contlTP	3
#define  dustTP		4
#define  vaporTP	-10	
#define IpcDIM		3
#define  naIP		0
#define	 nh4IP		1
#define  hIP		2
#define IpaDIM		5
#define	 clIP		0
#define	 so4IP		1
#define  hso4IP 	2
#define  ohIP		3
#define  no3IP          4

/*EMPIRICAL PARAMETERS*/
#define dmsINIT		5. /*ppt, Pandis et al., 1994*/
#define dmsFLUX		2./*umole/m2/d*/
#define h2so4INIT	0./*ppt, Pandis et al., 1994*/
#define so2FLAG		0 /*SO2 data is given by file=1; else=0 calculated from
			  initial value*/
#define h2so4THRESH	1.e-20 /*g/cc don't call nucln.f for lower h2so4*/
#define	so4hso4FRAC	0.5   /*mole fraction of sulfate to bisulfate (Quinn 90)*/
#define zeroTHRESH	1.e-20 /* dimensionless definition of zero */
#define minLWC		1.0e0/*min LWC/section as a factor of dry section mass */
#define maxLWC		1.e5/*max LWC/section as a factor of dry section mass*/
#define tolLWC		0.01/*tolerance on LWC/section as a factor of dry section mass*/
#define LWCfactor	2.
#define naclFRAC	0.25  /*Pruppacher p.145 at RH80%*/

/*FUNCTION DEFINITIONS*/
#define LN		log
#define LOG		log10

/*CONSTANTS AND CONVERSION FACTORS*/
#define PI		3.14159265358979323846
#define HRperDAY	24.
#define SECperHR	3600.
#define airMW		28.964
#define RGAS		(8.31441*.01) /*gas const in mb m3 / (K mole) */
#define AVOG		6.02205e23 /*Avogadro's number: molec/mole */
#define CCperM3		1.e6
#define kBOLTZ		1.38066e-23 /*Boltzmann constant: kg*m2/sec2/K */
#define eCHRG		1.60219e-19 /*C charge per electron*/
#define vacPERM     	1.11265e-10 /*C2/J/m or 4*PI*Eo vacuum permittivity*/
#define ktoCGS		1.e7 /*Convert J to g*cm2/sec2 */
#define	perMIL		1.e-6
#define perBIL		1.e-9
#define perTRIL		1.e-12
#define MICperM		1.e6 /*microns per m */
#define CMperMIC	1.e-4
#define GRAVITY		9.8 /*m/s2 Earth's gravitational constant */
#define CMperM		1.e2
#define GperKG		1.e3
#define JperCAL		4.184
#define ZEROC		273.150 /*K*/
#define MBARperATM	1.0133e3 /*1atm=1013.3mbar*/
#define NM2perMBAR	1.e2 /*1N/m2= 1 kg/m/sec2=0.01 mbar*/
#define NM2perTORR	133.3 /*1N/m2= 1 kg/m/sec2=0.01 mbar*/
#define NMperM		1.e9
#define TREF		298.15 /*ref. temp. for Pitzer Parameters*/

/*VARIABLE TYPE DEFINITIONS*/
typedef	char 				word24[24];
typedef double 				aryDp[DpDIM];
typedef double 				aryDpSp[DpDIM][SpDIM];
typedef double 				aryDpSpTp[DpDIM][SpDIM][TpDIM];
typedef double 				arySp[SpDIM];
typedef double				aryTp[TpDIM];
typedef double				aryDpTp[DpDIM][TpDIM];
typedef double				arySpTp[SpDIM][TpDIM];
struct dataProf {double alt; 
		 double temp;   /* Ambient temperature  (K) */
		 double pres;   /* Ambient pressure (mbar) */
		 double dewt;  /* dewpoint temperature  */ };  
struct dataInit	{int    index;
		 double wind;   /* WindSpeed (m/s)  */
   		 double updraft; /* Vertical velocity (m/s) */
		 double height;  /* Boundary layer height (m) */
                 double base; /* Cloud base height (m) */
		 double maxlwc;
		 double so2;	/* SO2 measured (ppt)->g/cm3 */
		 double ohestd; } 	;
extern word24 	CvNAME[SpDIM];

/*FUNCTION DECLARATIONS*/
/*rates.c*/
extern double Rflux( int, arySp, double, struct dataProf,struct dataInit);
extern double Rdepn(int,int, int, aryDpSpTp, double, struct dataProf,
 struct dataInit);
extern double RcoagAdd(int, int, int, aryDpSpTp,aryDpTp,struct dataProf);
extern double RcoagSub(int, int, int, aryDpSpTp,aryDpTp,struct dataProf);
extern double  CoagCoef(double, double, double, double, struct dataProf);
extern double Rnucl(int,int,int, aryTp,struct dataProf);
extern double  wetNuclMass(arySp,struct dataProf);
extern double Rgrow(aryDpSpTp,aryDpTp);
extern double Rcond(int,int,int,int,aryDpSpTp,arySp,double, struct dataProf);
extern double Rsalt(int,int,int,aryDp,struct dataProf, struct dataInit);
extern double Rcomb(int,int);

/*nucln.f*/
extern void h2so4nucleation_(double *, double *, double *, double *,double *,  
 double *);
extern void h2so4condensation_(double *, double *, double *, double *,double *, 
 double *,double *);

/*properties.c*/
extern double min(double, double);
extern double max(double, double);
extern double dryDp(float);
extern double wetDp(int, int, double,aryDpSpTp,double);
extern double wetPDensity (int, int, double,aryDpSpTp);
extern double wetPMass(int, int, double,aryDpSpTp, double);
extern double wetSMass(int, int, double,aryDpSpTp);
extern double SeaSaltDensity();
extern double dryPDensity (int, int, aryDpSpTp);
extern double dryPMass(int, int, aryDpSpTp);
extern double drySMass(int, int, aryDpSpTp);
extern double AirMeanPath (struct dataProf );
extern double AirViscosity (struct dataProf );
extern double SpDensity (int);
extern double MolWt (int);
extern double h2oSatWv (double);
extern double RelHumidity(struct dataProf,struct dataInit, aryDpSpTp, double, double);
extern double actRelHumidity(struct dataProf,struct dataInit, aryDpSpTp, double, double);
extern double LiqWaterContent(aryDpSpTp);
extern double Activity (int,int,int,double,aryDpSpTp,double,struct dataProf);
extern double Equilibrium (double,double,int,int,aryDpSpTp,double, struct dataProf);
extern double Surface(int,int,double,aryDpSpTp,double,struct dataProf);


/*initialize.c*/
extern void ReadSaltNp(aryDp);
extern void CalcNp(aryDpTp, aryDpSpTp);
extern void ReadMp(aryDpSpTp);
extern int ReadCv(arySp);
extern struct dataProf *ReadMet(int *);
extern struct dataInit *ReadInit(int *);

/*report.c*/
extern void ReportCv(int,double,arySp);
extern void ReportMp(int,double,aryDpSpTp);
extern void ReportNp(int,double,aryDpTp);
extern void ReportMet(int,double,double,double,double,double,double, double, double);
extern void ReportAct(int, double, double, double, double, double, double, double, double);
extern void Reportrate2(int, double, double, double);

/*zbrentlmr.c*/
extern double zbrent(double (*)(double,double,int,int,aryDpSpTp, double,struct dataProf), double, double, double, double,int,int,aryDpSpTp,double,struct dataProf);


